/***************************************************************************/
/** @file     skymap.cpp
  * @brief    The skymap window (implementation).
  *
  * @b begin      : Fri Nov 12 1999<br>
  * @b copyright  : &copy; 1999-2001,2002 by Gordon Machel<br>
  * @b email      : gmachel@users.sourceforge.net
  *
  * The skymap window is actually composed of two parts: a widget displaying
  * the skmap picture and a scroll view object containing the aforementioned
  * widget as a child. This guarantees efficient and flicker-free scrolling.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  **************************************************************************/
#include <kapp.h>
#include <klocale.h>
#include <kpopupmenu.h>
#include <kiconloader.h>

#include <qpainter.h>
#include <qpixmap.h>
#include <qscrollview.h>
#include <qcursor.h>

#include "ksetiwatch.h"
#include "skymap.h"
#include "datainfo.h"
#include "setiloc.h"

////////////////////////////////////////////////////////////////////
// SkyPic methods

SkyPic::SkyPic(QWidget* parent, const char*) : QWidget(parent)
{
list = dynamic_cast<SkyMap*>(parent)->skyposList();

pic_options = new KPopupMenu(this, "Skymap");
pic_options->setCheckable(true);
pic_options->insertItem(i18n("Show Text"), ShowText);
pic_options->insertSeparator();
pic_options->insertItem(i18n("Small Marker"), SmallMarker);
pic_options->insertItem(i18n("Normal Marker"), NormalMarker);
pic_options->insertItem(i18n("Large Marker"), LargeMarker);
pic_options->insertSeparator();
pic_options->insertItem(i18n("Fill Marker"), FillMarker);
connect(pic_options, SIGNAL(activated(int)), SLOT(handlePopupCommand(int)));
connect(pic_options, SIGNAL(aboutToShow()), SLOT(checkPopupStatus()));
}

/*------------------------------------------------------------------------ */
SkyPic::~SkyPic()
{
}

/*------------------------------------------------------------------------ */
void SkyPic::paintEvent(QPaintEvent*)
{
QPainter p;
QBrush bru;
QPen pen;
QString id;
SkyPos *pos;
int x, y;

p.begin( this );
for( pos=list->first(); pos != 0; pos=list->next() )
  {
  pen.setColor(pos->color);
  bru.setColor(pos->color);
  if(globalopts->FillMarker)
    bru.setStyle(SolidPattern);
  else
    bru.setStyle(NoBrush);
  p.setPen(pen);
  p.setBrush(bru);
  x = (int)(484.0 - (pos->ra*965.0)/24.0);
  if(x < 0) x = 965 + x;
  y = (int)(241.0 - (pos->dec*482.0)/180.0);
  int ms = globalopts->MarkerSize;
  p.drawEllipse(x-ms*3, y-ms*3, ms*6, ms*6);
  if(globalopts->ShowText)
    {
    id = pos->location + "\n" + pos->time_recorded;
    QRect r = p.boundingRect(0, 0, 10000, 10000, 0, (const char *)id);
    p.drawText(x-(r.width()/2), y-r.height()-ms*3-1,
               r.width(), r.height(), AlignCenter|DontClip, id);
    }
  }
p.end();
}

/*------------------------------------------------------------------------ */
void SkyPic::mousePressEvent(QMouseEvent *e)
{
if(e->button() == RightButton)
  {
  pic_options->popup(QCursor::pos());
  }
}

/*------------------------------------------------------------------------ */
void SkyPic::checkPopupStatus()
{
pic_options->setItemChecked(ShowText, globalopts->ShowText);
pic_options->setItemChecked(FillMarker, globalopts->FillMarker);
for(int i=SmallMarker;i<=LargeMarker;i++)
  {
  pic_options->setItemChecked(i, false);
  }
pic_options->setItemChecked(globalopts->MarkerSize+SmallMarker-1, true);
}

/*------------------------------------------------------------------------ */
void SkyPic::handlePopupCommand(int id)
{
switch(id)
  {
  case ShowText: // handle the text display
    globalopts->ShowText = !globalopts->ShowText;
    break;
  case FillMarker: // fill marker?
    globalopts->FillMarker = !globalopts->FillMarker;
    break;
  default: // marker size
    if(id >= SmallMarker && id <= LargeMarker)
      globalopts->MarkerSize = id - SmallMarker + 1;
    break;
  }

update();
}


////////////////////////////////////////////////////////////////////
// SkyMap methods		
SkyMap* SkyMap::map = 0;

SkyMap::SkyMap(QWidget* parent, const char*)
       : QScrollView(parent)
{
SeLi.setAutoDelete(true);

// load the skymap
starmap = UserIcon("starmap");

setCaption(i18n("Skymap (right click for menu)"));
setMinimumSize(100, 100);

QSize s(starmap.size());

pic = new SkyPic(this);
pic->setBackgroundPixmap(starmap);
pic->resize(s);
this->addChild(pic);

QWidget* d = QApplication::desktop();
int w = d->width();                   // returns screen width
int h = d->height();                  // returns screen height
int w_sm(0), h_sm(0);                 // width and height of starmap window
// +4 had to be added to avoid scroll bars in max mode (Qt 2.2)
if(s.width() < w) w_sm = s.width()+4; else {w_sm = w - 8; h_sm = 16;}
if(s.height() < h) h_sm += s.height()+4; else h_sm = h;	
resize(w_sm, h_sm);
setMaximumSize(w_sm, h_sm);	
}

/*------------------------------------------------------------------------ */
SkyMap::~SkyMap()
{
SeLi.clear();
}

/*------------------------------------------------------------------------ */
SkyMap* SkyMap::instance()
{
if(map == 0)
  map = new SkyMap();
return(map);
}

/*------------------------------------------------------------------------ */
SkyMap* SkyMap::showMap()
{
SkyMap* m = instance();
if(m)
  {
  m->show();
  if(m->isActiveWindow() == false)
    {
    if(m->isMinimized())
      {
      m->hide();
      m->showNormal();
      }
    m->raise();
    }
  }
return(m);
}

/*------------------------------------------------------------------------ */
void SkyMap::addLocation(SetiLoc* loc)
{
if(!loc->timeRecorded().isEmpty())
  {
  SkyPos *spos = new SkyPos;

  // fill the SkyPos structure
  spos->location = loc->description();
  spos->time_recorded = loc->timeRecordedString();
  spos->color = loc->color();
  spos->ra = loc->startRA();
  spos->dec = loc->startDec();

  // and add it to the list
  SeLi.append(spos);
  pic->update();
  }

connect(loc, SIGNAL(newWorkUnit(WorkUnitData, const QString&, const QColor&)),
     this, SLOT(updateLocation(WorkUnitData, const QString&, const QColor&)));
}

/*------------------------------------------------------------------------ */
void SkyMap::addLocation(SetiLoc* loc, const QString& time_rec,
                         double ra, double dec)
{
SkyPos *spos = new SkyPos;

// fill the SkyPos structure
spos->location = loc->description();
spos->time_recorded = time_rec;
spos->color = loc->color();
spos->ra = ra;
spos->dec = dec;

// and add it to the list
SeLi.append(spos);
pic->update();
}

/*------------------------------------------------------------------------ */
void SkyMap::clearMap()
{
SeLi.clear();
pic->setBackgroundPixmap(*pic->backgroundPixmap());
}

/*------------------------------------------------------------------------ */
void SkyMap::updateLocation(WorkUnitData wud, const QString& locname,
                            const QColor& col)
{
for(SkyPos* pos=SeLi.first(); pos != 0; pos=SeLi.next() )
  {
  if(pos->location == locname)
    {
    pos->time_recorded = wud.time_recorded;
    pos->color = col;
    pos->ra = wud.start_ra;
    pos->dec = wud.start_dec;
    pic->update();
    break;
    }
  }
}

#include "skymap.moc"

