/***************************************************************************/
/** @file     skymap.h
  * @brief    The skymap window  (declaration).
  *
  * @b begin      : Fri Nov 12 1999<br>
  * @b copyright  : &copy; 1999-2001,2002 by Gordon Machel<br>
  * @b email      : gmachel@users.sourceforge.net
  *
  * The skymap window is actually composed of two parts: a widget displaying
  * the skmap picture and a scroll view object containing the aforementioned
  * widget as a child. This guarantees efficient and flicker-free scrolling.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  **************************************************************************/
#ifndef SKYMAP_H
#define SKYMAP_H

#include <qwidget.h>
#include <qscrollview.h>
#include <qlist.h>

#include "setidefs.h"

class KPopupMenu;
class QPixmap;
class SetiLoc;

/** A structure containing details about the work unit shown on the skymap. */
struct SkyPos {
    QString location;      /**< The name of the location the WU belongs to. */
    QString time_recorded; /**< The time and date the WU was recorded. */
    QColor  color;         /**< The color used for the marker. */
    double  ra;            /**< The position of the WU on the skymap (right
                                ascension). */
    double  dec;           /**< The position of the WU on the skymap
                                (declination). */
    };

/** The widget displaying the skymap.
  *
  * This widget is a child of the SkyMap class. Markers are drawn onto the
  * SkyPic widget indicating in which part of the sky the WUs have been
  * recorded.
  */
class SkyPic : public QWidget
{
Q_OBJECT

public:
  /** The constructor.
    *
    * @param parent The parent widget. If 0, this widget becomes a top-level
    *               widget.
    * @param name   A text that can be used to identify this widget.
    */
  SkyPic(QWidget* parent=0, const char* name=0);
  /** The destructor. */
  ~SkyPic();

protected: //methods
  /** Reimplemented for customized painting of the widget.
    *
    * @param e Contains event parameters of the paint event.
    */
  virtual void paintEvent(QPaintEvent *e);
  /** An event handler for mouse press events in the skymap window .
    *
    * Required to catch the right-mouse-button click for the context menu.
    *
    * @param e An object with parameters describing the mouse event.
    */
  virtual void mousePressEvent(QMouseEvent *e);

protected: //attributes	
  /** Identifiers for the menu items in the right-click popup menu. */
  enum popupIds {ShowText, SmallMarker, NormalMarker, LargeMarker, FillMarker};
  KPopupMenu*    pic_options;
  QList<SkyPos>* list;

protected slots:
  /** Prepares the right-click popup menu just before showing it.
    *
    * Checks if the popup menu commands are allowed or make sense in the
    * current situation, and enables or disables the menu items appropriately.
    */
  void checkPopupStatus();
  /** Executes the selected command from the right-click popup menu of the
    * Analysis list.
    *
    * @param id The unique ID number of the selected popup menu item. See
    *           SkyPic::popupIds for details.
    */
  void handlePopupCommand(int id);
};

/** The skymap scroll view.
  *
  * The scroll view contains a SkyPic widget displaying the skymap. By
  * default, the scroll view window tries to open itself maximized. If the
  * desktop is smaller than the size of the skymap, the window size is
  * adjusted appropriately.
  *
  * The SkyMap class is implemented as a singleton, which means that only one
  * instance of this class can be created during the run-time of the
  * application.
  */
class SkyMap : public QScrollView
{
Q_OBJECT

public:
  /** Returns a pointer to the SkyMap object.
    *
    * If the object doesn't exist, it is created.
    */
  static SkyMap* instance();
  /** Opens the skymap window.
    *
    * If the window is minimized or hidden, it is raised and brought to the
    * foreground.
    *
    * @return A pointer to the SkyMap object.
    */
  static SkyMap* showMap();
  /** Adds a location to the skymap.
    *
    * This methods adds the WU currently processed by %S@h location @a loc to
    * the list of displayed WU sky position.
    *
    * @param loc A pointer to the SetiLoc object.
    */
  void addLocation(SetiLoc* loc);
  /** Adds a location to the skymap.
    *
    * @overload
    *
    * @todo Replace @a loc with arguments for description and color.
    *
    * @param loc      A pointer to the SetiLoc object.
    * @param time_rec The time and date the WU was recorded.
    * @param ra       The position of the WU on the skymap (right ascension).
    * @param dec      The position of the WU on the skymap (declination).
    */
  void addLocation(SetiLoc* loc, const QString& time_rec, double ra, double dec);
  /** Clears the list of added locations and redraws the skymap. */
  void clearMap();
  /** Returns a pointer to the list of WU sky positions currently displayed. */
  QList<SkyPos>* skyposList() {return(&SeLi);}

protected: // methods
  /** The destructor. */
  virtual ~SkyMap();

protected slots:
  /** Updates a location with new WU data.
    *
    * @param wud     A structure containing the new WU data.
    * @param locname The descriptive name of the %S@h location.
    * @param col     The color used for the markers.
    */
  void updateLocation(WorkUnitData wud, const QString& locname,
                      const QColor& col);

protected: // attributes
  /** This list contains the sky position and additional information of all
    * WUs on the skymap. */
  QList<SkyPos> SeLi;
  /** A picture of the skymap. */
  QPixmap       starmap;
  /** A pointer to the widget displaying the skymap. */
  SkyPic*       pic;

private: //methods
  /** The constructor.
    *
    * Imlplemented as a private method because this class is implemented as
.   * a singleton
    *
    * @param parent The parent widget. If 0, this widget becomes a top-level
    *               widget.
    * @param name   A text that can be used to identify this widget.
    */
  SkyMap(QWidget* parent=0, const char* name=0);

private: // attributes
  /** A pointer to the one and only instance of this class. */
  static SkyMap* map;
};

#endif
